@@ -109,7 +109,7 @@ int  ReadHSpice(char *fname);
  *---------------------------------------------------------------
  */
 
-extern int asg_make_instance(Clientdata *, char *, int, int, int, int, float);
+extern int asg_make_instance(Clientdata *, char *, int, int, int, int);
 extern void asg_make_polygon(Clientdata *, int, int x[], int y[]);
 extern void asg_make_label(Clientdata *, u_char, int, int, char *);
 
