@@ -22,7 +22,7 @@
 /* 
  *----------------------------------------------------------------------
  * File 	psfigs.c
- * Author 	s.t.frezza (frezza@jupiter.ee.pitt.edu)
+ * Author 	s.t.frezza 
  * 
  * This file contains supporting post-script generation routines to produce 
  * figures for predefined schematic symbols
@@ -30,8 +30,10 @@
  *----------------------------------------------------------------------
  */
 #include <stdio.h>
-#include "ivf_format.h"
+#include "network.h"
 #include "route.h"
+#include "psfigs.h"
+#include "asg_module.h"
 
 #define NAME_OFFSET -1.1
 #define SIDE_CONVERT(n) ((n==0)?0:(n==1)?1:(n==2)?2:(n==3)?3:0)
@@ -221,93 +223,6 @@ int ps_print_mod(f,m)
 	}
     }
 }
-/*--------------------------------------------------------------------------------- */
-int xc_print_mod(areastruct, m)
-    Clientdata *areastruct;
-    module *m;
-{
-    int type, n, i;
-    float scale, arcLen, noOfExtensions, fractional, theta, mid_y, mid_x;
-    
-    if (m != NULL)
-    {
-	type = validate(m->type);
-	 scale = (float)(m->x_size)/(float)(ICON_SIZE * ICON_SIZE/2);
-	
-
-	xc_print_sym(areastruct, m->type, m->x_pos,  m->y_pos, m->x_size, m->y_size, 
-		     m->rot, m->name, scale);
-
-	if ((type == BLOCK) || (type == DONT_KNOW)) xc_attach_outerms(areastruct, m);
-	xc_attach_interms(areastruct, m);
-
-	/* Special case stuff for mulit-terminal ANSI AND, NAND, OR, NOR, XOR, and
-	   XNOR gates: */
-
-	if ((m->y_size > ICON_SIZE) && (type != BLOCK) && (type != DONT_KNOW))
-	{
-	    scale = (float)m->x_size/(float)(ICON_SIZE * ICON_SIZE/2);
-
-	    if ((type == AND) || (type == NAND))
-	    {   /* AND family - display a line to connect all inputs to the icon */
-		mid_x = (float)m->x_pos + (float)m->x_size / 2.0;
-
-		//code to be attached 
-	    }
-
-	    else if ((type == OR) || (type == NOR))
-	    {   /* OR family -- Add line to the 1st Back Curve:  */
-		mid_x = (float)m->x_pos + (float)m->x_size/2.0 - 0.4;
-		arcLen = (18.8232 * scale);
-		noOfExtensions = ((float)m->y_size - arcLen)/(2.0 * arcLen);
-		n = (int)noOfExtensions;
-		fractional = (noOfExtensions - (float)n) * arcLen;
-		for (i = n; i >= 0; i--)
-		{	/* put down one arc, or fraction thereof */
-		    theta = (i > 0) ? 28.072 : 
-		            (180.0/PI) * asin(fractional/(scale * 20)) - 28.072;
-		    if (theta > (-28.072 * 1.10))	/* 10% margin */
-		    {
-			mid_y = (float)m->y_pos + (float)m->y_size/2.0 + 
-			        (float)(n - i + 1) * arcLen;
-			
-			mid_y = (float)m->y_pos + (float)m->y_size/2.0 - 
-			        (float)(n - i + 1) * arcLen;
-			//code to be attached
-		    }
-		}
-	    }
-	    else if ((type == XOR) || (type == XNOR))
-	    {   /* OR family -- Add line to the 2nd Back Curve:  */
-		mid_x = (float)m->x_pos + (float)m->x_size/2.0;
-		arcLen = (18.8232 * scale);
-		noOfExtensions = ((float)m->y_size - arcLen)/(2.0 * arcLen);
-		n = (int)noOfExtensions;
-		fractional = (noOfExtensions - (float)n) * arcLen;
-		for (i = n; i >= 0; i--)
-		{	/* put down one arc, or fraction thereof */
-		    theta = (i > 0) ? 28.072 : 
-		            (180.0/PI) * asin(fractional/(scale * 20)) - 28.072;
-		    if (theta > (-28.072 * 1.10))	/* 10% margin */
-		    {
-			mid_y = (float)m->y_pos + (float)m->y_size/2.0 + 
-			        (float)(n - i + 1) * arcLen;
-			/*fprintf(f, "newpath %f %f %f %f %f arc stroke\n", 
-				mid_x - scale * 31.5, mid_y, scale * 20.0, 
-				-28.072, theta);*/
-			mid_y = (float)m->y_pos + (float)m->y_size/2.0 - 
-			        (float)(n - i + 1) * arcLen;
-			//code to be attached
-		    }
-		}
-	    }
-	    /* height of arc in Y is 2*rad*sin(28.072) = 40*scale*.47058 = 
-	       scale * 18.8232. */
-	}
-    }
-}
-
-
 /*--------------------------------------------------------------------------------- */
 int ps_attach_interms(f, m)
     FILE *f;
@@ -659,9 +574,6 @@ int ps_print_sym(f, sym, x, y, x_dis, y_dis, rot, name,scale)
     int x, y, x_dis, y_dis, rot;
 	float scale;
 {
-    
-    
-
     /* print the symbol that goes with this type: */
     switch(validate(sym))
     {
@@ -744,21 +656,7 @@ int ps_print_sym(f, sym, x, y, x_dis, y_dis, rot, name,scale)
      return(1);
 }
 
-/*--------------------------------------------------------------------------------- */
-
-int xc_print_sym(areastruct, sym, x, y, x_dis, y_dis, rot, name, scale)
-    Clientdata *areastruct;
-    char *sym, *name;
-    int x, y, x_dis, y_dis, rot;
-    float scale;
-{
-    asg_make_instance(areastruct, sym, x, y, x_dis, y_dis, scale);
-    asg_make_label(areastruct, NORMAL, x * 15 , y * 15, name);
-
-    return(1);
-}
-
-/*--------------------------------------------------------------------------------- */
+/*---------------------------------------------------------------------- */
 
 int ps_put_label(f, str, x, y)
     FILE *f;
@@ -778,37 +676,8 @@ int ps_put_int(f, i, x, y)
     fprintf(f,"%f %f moveto (%d) show\n", x,y,i);
 }
 
-/*--------------------------------------------------------------------------------- */
 
-int validate(s)
-    char *s;
-{			
-    /* SEE "ivf_format.h" and "network.h" */
-    if (!strcmp(s, GATE_AND_STR)) return AND;
-    if (!strcmp(s, GATE_OR_STR)) return OR;
-    if (!strcmp(s, GATE_NAND_STR)) return NAND;
-    if (!strcmp(s, GATE_NOR_STR)) return NOR;
-    if (!strcmp(s, GATE_PMOS_STR)) return PMOS;
-    if (!strcmp(s, GATE_CAPC_STR)) return CAPC;
-    if (!strcmp(s, GATE_RESTR_STR)) return RESTR;
-    if (!strcmp(s, GATE_INDR_STR)) return INDR;
-    if (!strcmp(s, GATE_XOR_STR)) return XOR;
-    if (!strcmp(s, XNOR_GATE)) return XNOR;
-    if (!strcmp(s, GATE_NOT_STR)) return NOT_;
-    if (!strcmp(s, GATE_INVNOT_STR)) return INVNOT_;
-    if (!strcmp(s, GATE_NULL_STR)) return BUFFER;
-    if (!strcmp(s, BUFFER_GATE)) return BUFFER;
-    if (!strcmp(s, INPUT_TERM)) return INPUT_SYM;
-    if (!strcmp(s, OUTPUT_TERM)) return OUTPUT_SYM;
-    if (!strcmp(s, INOUT_TERM)) return INOUT_SYM;
-    if (!strcmp(s, GENERIC_BLOCK)) return BLOCK;
-    if (!strcmp(s, GATE_NMOS_STR)) return NMOS;
-    if (!strcmp(s, GATE_VAMP_STR)) return VAMP;
-    if (!strcmp(s, GATE_MSFET_STR)) return MSFET;
-    else return(DONT_KNOW);
-}
-/* Also adjust "set_default_object_size" whenever this is modified. */
-/*--------------------------------------------------------------------------------- */
+/*----------------------------------------------------------------------- */
 int ps_print_outterm(f, x, y, x_size, y_size)
     FILE *f;
     int x, y, x_size, y_size;
