@@ -507,7 +507,7 @@ void loadlgf(int mode)
 
    for (iolabel = topobject->plist; iolabel < topobject->plist +
 	   topobject->parts; iolabel++)
-      if ((*iolabel)->type == LABEL) {
+      if (IS_LABEL(*iolabel)) {
          if (TOLABEL(iolabel)->rotation == 500) {
 	    genericptr *tmplabel;
 
