@@ -56,6 +56,8 @@ extern char  _STR[150], _STR2[250];
 #define BUTTON1 (Button1Mask << 16)
 #define BUTTON2 (Button2Mask << 16)
 #define BUTTON3 (Button3Mask << 16)
+#define BUTTON4 (Button4Mask << 16)
+#define BUTTON5 (Button5Mask << 16)
 
 /*--------------------------------------------------------------*/
 /* This list declares all the functions which can be bound to   */
@@ -83,9 +85,9 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Exchange", "Copy", "Move", "Join", "Unjoin", "Spline", "Edit",
    "Undo", "Redo", "Select Save", "Unselect", "Dashed", "Dotted",
    "Solid", "Prompt", "Dot", "Wire", "Start", "Finish", "Cancel",
-   "Nothing", "Exit",
-   "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label",
-   "Connectivity", "Sim", "SPICE", "PCB", "SPICE Flat",
+   "Nothing", "Exit", "Netlist", "Swap", "Pin Label", "Pin Global",
+   "Info Label", "Graphic", "Connectivity", "Sim", "SPICE", "PCB",
+   "SPICE Flat",
    NULL			/* sentinel */
 };
 
@@ -251,6 +253,8 @@ int string_to_key(const char *keystring)
 	       case '1': keywstate = (Button1Mask << 16); break;
 	       case '2': keywstate = (Button2Mask << 16); break;
 	       case '3': keywstate = (Button3Mask << 16); break;
+	       case '4': keywstate = (Button4Mask << 16); break;
+	       case '5': keywstate = (Button5Mask << 16); break;
 	    }
          }
 	 else {
@@ -340,6 +344,8 @@ char *key_to_string(int keywstate)
       if (kmod & Button1Mask) strcat(kptr, "Button1");
       else if (kmod & Button2Mask) strcat(kptr, "Button2");
       else if (kmod & Button3Mask) strcat(kptr, "Button3");
+      else if (kmod & Button4Mask) strcat(kptr, "Button4");
+      else if (kmod & Button5Mask) strcat(kptr, "Button5");
       else {
 	 kptr[0] = '0';
 	 kptr[1] = 'x';
