@@ -1581,6 +1581,9 @@ void renamepage(short pagenumber)
       Fprintf(stderr, "Error:  No Button Widget named \"%9s\"\n", bname);
 #endif
 
+   /* The GUI has been taken care of, but the page itself might not exist */
+   if (!thisinst) return;
+
    /* If the page had a master but now has a different	*/
    /* name, disconnect the schematics.			*/
 
@@ -1607,7 +1610,6 @@ void renamepage(short pagenumber)
 	 break;
       }
    }
-
 }
 
 /*--------------------------------------------------------------*/
