@@ -230,6 +230,11 @@ menustruct RotateMenu[] = {
 	{"Rotate CCW 1", action(startrotate, Number(-1))},
 };
 
+menustruct LineMenu[] = {
+	{"Global Linewidth", action(getwirewidth, NULL)},
+	{"Wire Linewidth", action(getwwidth, NULL)},
+};
+
 menustruct GridMenu[] = {
 	{"Grid", setaction(toggle, &areastruct.gridon)},
 	{"Axes", setaction(toggle, &areastruct.axeson)},
@@ -312,7 +317,7 @@ menustruct OptionMenu[] = {
 #endif
 	{"Grid", submenu(GridMenu)},
 	{"Snap-to", submenu(SnapMenu)},
-	{"Global Linewidth", action(getwirewidth, NULL)},
+	{"Linewidth", submenu(LineMenu)},
 	{"Elements", submenu(PolyMenu)},
 	{" ", noaction},
 	{"Help!", action(starthelp, NULL)},
