@@ -14,6 +14,8 @@ extern void flush_redo_stack();
 extern void truncate_undo_stack();
 extern void free_undo_record(Undoptr);
 extern void free_redo_record(Undoptr);
+extern stringpart *get_original_string(labelptr);
+extern short *recover_selectlist(Undoptr);
 #ifndef TCL_WRAPPER
 extern void undo_call(xcWidget, caddr_t, caddr_t);
 extern void redo_call(xcWidget, caddr_t, caddr_t);
@@ -27,24 +29,55 @@ extern Tk_EventProc *xctk_drag(ClientData, XEvent *);
 extern Tk_EventProc *xctk_draglscroll(ClientData, XEvent *);
 extern Tk_EventProc *xctk_dragfilebox(ClientData, XEvent *);
 extern int XcTagCallback(Tcl_Interp *, int, Tcl_Obj *CONST objv[]);
-extern Tcl_Obj *evaluate_raw(oparamptr);
+extern Tcl_Obj *evaluate_raw(oparamptr, int *);
 extern char *TCL_to_PS(char *);
 #endif
 
-extern char *evaluate_expr(oparamptr);
+extern char *evaluate_expr(oparamptr, objinstptr);
+
+/* from opengl.c:  compatibility functions for OpenGL/X11 */
+
+#ifdef OPENGL
+extern void SetForeground(Display *, GC, int);
+extern void SetBackground(Display *, GC, int);
+extern void SetFunction(Display *, GC, int);
+extern void SetLineAttributes(Display *, GC, float, int, int, int);
+extern void SetDashes(Display *, GC, int, char dashlist[], int);
+extern void SetStipple(Display *, GC, int);
+extern void SetFillStyle(Display *, GC, int);
+extern void DrawLine(Display *, Window, GC, int, int, int, int);
+extern void DrawLines(Display *, Window, GC, XPoint *, int, int);
+extern void DrawPoint(Display *, Window, GC, int, int);
+extern void FillPolygon(Display *, Window, GC, XPoint *, int, int, int);
+#endif
 
 /* from elements.c: */
 
-extern void copyalleparams(genericptr, genericptr);
-extern void copyparams(objinstptr, objinstptr);
-extern void instancedefaults(objinstptr, objectptr, int, int);
-extern void labeldefaults(labelptr, u_char, int, int);
-extern void poly_add_point(polyptr, XPoint *);
+/* element constructor functions */
+extern labelptr new_label(objinstptr, stringpart *, int, int, int);
+extern labelptr new_simple_label(objinstptr, char *, int, int, int);
+extern labelptr new_temporary_label(objectptr, char *, int, int);
+extern polyptr new_polygon(objinstptr, pointlist *, int);
+extern splineptr new_spline(objinstptr, pointlist);
+extern arcptr new_arc(objinstptr, int, int, int);
+extern objinstptr new_objinst(objinstptr, objinstptr, int, int);
+
+/* element destructor function */
+extern void remove_element(objinstptr, genericptr);
+
+/* functions to set default values for element types */
 extern void polydefaults(polyptr, int, int, int);
 extern void splinedefaults(splineptr, int, int);
 extern void arcdefaults(arcptr, int, int);
 extern void pathdefaults(pathptr, int, int);
+extern void instancedefaults(objinstptr, objectptr, int, int);
+extern void labeldefaults(labelptr, u_char, int, int);
+
+extern void poly_add_point(polyptr, XPoint *);
 extern void drawdot(int, int);
+extern void copyalleparams(genericptr, genericptr);
+extern void copyparams(objinstptr, objinstptr);
+
 extern void startspline(xcWidget, caddr_t, caddr_t);
 extern void starttext(xcWidget, pointertype, caddr_t);
 extern void textbutton(u_char, XButtonEvent *);
@@ -52,6 +85,17 @@ extern void charreport(labelptr);
 extern void labeltext(int, char *);
 extern void textreturn();
 extern void rejustify(short);
+extern void finddir(polyptr);
+extern void reversepoints(XPoint *, short);
+extern void reversefpoints(XfPoint *, short);
+extern void freeparts(short *, short);
+extern void removep(short *, short);
+extern void unjoin();
+extern labelptr findlabelcopy(labelptr, stringpart *);
+extern Boolean neartest(XPoint *, XPoint *);
+extern void join();
+
+/* interactive manipulation of elements */
 extern void splinebutton(XButtonEvent *);
 extern void trackspline(xcWidget, caddr_t, caddr_t);
 extern void startarc(xcWidget, caddr_t, caddr_t);
@@ -62,17 +106,8 @@ extern void boxbutton(XButtonEvent *);
 extern void trackbox(xcWidget, caddr_t, caddr_t);
 extern void trackwire(xcWidget, caddr_t, caddr_t);
 extern void startwire(XPoint);
-extern void finddir(polyptr);
 extern void trackpoly(xcWidget, caddr_t, caddr_t);
 extern void setendpoint(short *, short, XPoint **, XPoint *);
-extern void reversepoints(XPoint *, short);
-extern void reversefpoints(XfPoint *, short);
-extern void freeparts(short *, short);
-extern void removep(short *, short);
-extern void unjoin();
-extern labelptr findlabelcopy(labelptr, stringpart *);
-extern Boolean neartest(XPoint *, XPoint *);
-extern void join();
 extern void wirebutton(XButtonEvent *);
 
 /* from events.c: */
@@ -99,6 +134,7 @@ extern void endvbar(xcWidget, caddr_t, XButtonEvent *);
 extern void zoombox(xcWidget, caddr_t, caddr_t);
 extern void zoomin(xcWidget, caddr_t, XButtonEvent *);
 extern void zoomout(xcWidget, caddr_t, XButtonEvent *);
+extern void warppointer(int, int);
 extern void panbutton(u_int, XButtonEvent *);
 extern void zoominrefresh(xcWidget, caddr_t, XButtonEvent *);
 extern void zoomoutrefresh(xcWidget, caddr_t, XButtonEvent *);
@@ -146,10 +182,11 @@ extern void startedit(xcWidget, caddr_t, caddr_t);
 extern void startdelete(xcWidget, caddr_t, caddr_t);
 extern void reviseselect(short *, int, short *);
 extern void deletebutton(XButtonEvent *);
-extern short *xc_undelete(objinstptr, objectptr, short);
+extern short *xc_undelete(objinstptr, objectptr, short, short *);
 extern objectptr delete_element(objinstptr, short *, int, short);
 extern void printname(objectptr);
 extern Boolean checkname(objectptr);
+extern char *checkpostscriptname(char *, int *, objectptr);
 extern objectptr finddot();
 extern void movepoints(genericptr *, short, short);
 extern void domakeobject(xcWidget, caddr_t);
@@ -157,6 +194,7 @@ extern void selectsave(xcWidget, caddr_t, caddr_t);
 extern void arceditpush(arcptr);
 extern void splineeditpush(splineptr);
 extern void polyeditpush(polyptr);
+extern pointlist copypoints(pointlist, int);
 extern void labelcopy(labelptr, labelptr);
 extern void arccopy(arcptr, arcptr);
 extern void polycopy(polyptr, polyptr);
@@ -196,7 +234,7 @@ extern void importspice();
 #endif
 
 extern char *ridnewline(char *);
-extern void free_single(genericptr *);
+extern void free_single(genericptr);
 extern void reset(objectptr, short);
 extern void pagereset(short);
 extern void initmem(objectptr);
@@ -232,6 +270,13 @@ extern char *varscan(objectptr, char *, short *, genericptr, u_char);
 extern char *varfscan(objectptr, char *, float *, genericptr, u_char);
 extern objinstptr addtoinstlist(int, objectptr, Boolean);
 extern Boolean objectread(FILE *, objectptr, short, short, short, char *, int);
+void importfromlibrary(short, char *, char *);
+objectptr *new_library_object(short, char *, objlistptr *);
+Boolean library_object_unique(short, objectptr, objlistptr);
+void add_object_to_library(short, objectptr);
+u_char *find_delimiter(u_char *);
+char standard_delimiter_end(char);
+
 #ifdef TCL_WRAPPER
 void setfile(char *, int);
 extern xcTimeOutProc savetemp(caddr_t);
@@ -248,7 +293,7 @@ extern char *nosprint(char *);
 extern Boolean xc_tilde_expand(char *);
 extern short writelabel(FILE *, stringpart *, short *);
 extern char *writesegment(stringpart *, float *, int *);
-extern void writelabelsegs(FILE *, short *, stringpart *);
+extern int writelabelsegs(FILE *, short *, stringpart *);
 extern void printobjects(FILE *, objectptr, objectptr **, short *, int);
 extern void printrefobjects(FILE *, objectptr, objectptr **, short *);
 extern void printpageobject(FILE *, objectptr, short, short);
@@ -279,12 +324,13 @@ extern void findsplinepos(splineptr, float, XPoint *, int *);
 extern void ffindsplinepos(splineptr, float, XfPoint *);
 extern float findsplinemin(splineptr, XPoint *);
 extern short closepoint(polyptr, XPoint *);
+extern short closedistance(polyptr, XPoint *);
 extern void updateinstparam(objectptr);
 extern short checkbounds();
 extern void window_to_user(short, short, XPoint *);
 extern void user_to_window(XPoint, XPoint *);
 extern float UTopScale();
-extern short UTopTransScale(float);
+extern float UTopTransScale(float);
 extern XPoint UGetCursor();
 extern XPoint UGetCursorPos();
 extern void u2u_snap(XPoint *);
@@ -309,7 +355,7 @@ extern void InvTransformPoints(XPoint *, XPoint *, short, XPoint, float,
 extern void manhattanize(XPoint *, polyptr);
 extern void bboxcalc(short, short *, short *);
 extern void calcextents(genericptr *, short *, short *, short *, short *);
-extern void objinstbbox(objinstptr, XPoint *);
+extern void objinstbbox(objinstptr, XPoint *, Boolean);
 extern void labelbbox(labelptr, XPoint *, objinstptr);
 extern void calcbboxsingle(genericptr *, objinstptr, short *, short *, short *, short *);
 extern Boolean object_in_library(short, objectptr);
@@ -513,58 +559,65 @@ extern Tcl_Obj *tcltoplevel(objinstptr);
 extern Boolean RemoveFromNetlist(objectptr, genericptr);
 extern labelptr NetToLabel(int, objectptr);
 extern void NameToPosition(objinstptr, labelptr, XPoint *);
+extern XPoint *NetToPosition(int, objectptr);
 extern void invalidate_netlist(objectptr);
+extern void remove_netlist_element(objectptr, genericptr);
 extern int updatenets(objinstptr);
 extern void createnets(objinstptr);
 extern void destroynets();
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
-extern void gennetlist(objectptr, objinstptr);
-extern void gencalllist(objectptr);
+extern void gennetlist(objinstptr);
+extern void gencalls(objectptr);
 extern void search_on_siblings(objinstptr, objinstptr, pushlistptr,
 		short, short, short, short);
-extern int is_resolved(genericptr *, pushlistptr, objectptr *);
+extern void copy_bus(Genericlist *, Genericlist *);
+extern Genericlist *is_resolved(genericptr *, pushlistptr, objectptr *);
 extern Boolean highlightnet(objectptr, objinstptr, int, u_char);
 extern int pushnetwork(pushlistptr, objectptr);
 extern int onsegment(XPoint *, XPoint *, XPoint *);
 extern Boolean neardist(long);
 extern Boolean nearpoint(XPoint *, XPoint *);
-extern int make_tmp_pin(objectptr, XPoint *);
-extern Boolean searchconnect(XPoint *, int, objinstptr);
-extern int translateup(int, objectptr, objectptr, objinstptr);
-extern void addpoly(objectptr, polyptr, int);
+extern int searchconnect(XPoint *, int, objinstptr, int);
+extern Genericlist *translateup(Genericlist *, objectptr, objectptr, objinstptr);
+extern Genericlist *addpoly(objectptr, polyptr, Genericlist *);
 extern long zsign(long, long);
-extern void mergecleanup(objectptr);
-extern void mergenets(objectptr, int, int);
+extern Boolean mergenets(objectptr, Genericlist *, Genericlist *);
 extern void removecall(objectptr, CalllistPtr);
-extern void addpin(objectptr, labelptr, int);
+extern Genericlist *addpin(objectptr, objinstptr, labelptr, Genericlist *);
 extern void addcall(objectptr, objectptr, objinstptr);
-extern Boolean addport(objectptr, int);
-extern void addportcall(objectptr, objinstptr, objectptr, int, int);
+extern void addport(objectptr, Genericlist *);
+extern Boolean addportcall(objectptr, Genericlist *, Genericlist *);
 extern void makelocalpins(objectptr, CalllistPtr, char *);
 extern int porttonet(objectptr, int);
-extern int pointtonet(objectptr, XPoint *);
 extern stringpart *nettopin(int, objectptr, char *);
-extern int pintonet(objectptr, objinstptr, labelptr);
-extern int nametonet(objectptr, objinstptr, char *);
+extern Genericlist *pointtonet(objectptr, objinstptr, XPoint *);
+extern Genericlist *pintonet(objectptr, objinstptr, labelptr);
+extern Genericlist *nametonet(objectptr, objinstptr, char *);
+extern Genericlist *new_tmp_pin(objectptr, XPoint *, char *, char *, Genericlist *);
+extern Genericlist *make_tmp_pin(objectptr, objinstptr, XPoint *, Genericlist *);
+extern void resolve_devnames(objectptr);
+extern void resolve_devindex(objectptr, Boolean);
 extern char *parseinfo(objectptr, CalllistPtr, char *, char *, Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
 extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
 extern void writeglobals(objectptr, FILE *);
 extern void writehierarchy(objectptr, CalllistPtr, FILE *);
 extern void writenet(objectptr, char *, char *);
-extern void writepcb(struct Ptab **, objectptr, CalllistPtr, char *, char *);
+extern Boolean writepcb(struct Ptab **, objectptr, CalllistPtr, char *, char *);
 extern void outputpcb(struct Ptab *, FILE *);
 extern void freepcb(struct Ptab *);
-extern void freepolylist(NetlistPtr);
-extern void freelabellist(NetlistPtr);
-extern void freeports(CalllistPtr);
+extern void freegenlist(Genericlist *);
+extern void freepolylist(PolylistPtr *);
+extern void freelabellist(LabellistPtr *);
+extern void freecalls(CalllistPtr);
 extern void freenets(objectptr);
 extern void freetemplabels(objectptr);
 extern void freeglobals();
-extern int cleartraversed(objectptr);
-extern int checkvalid(objectptr);
-extern void clearpins(objectptr);
+extern void destroynets(objectptr);
+extern int  cleartraversed(objectptr);
+extern int  checkvalid(objectptr);
+extern void clearlocalpins(objectptr);
 
 /* from parameter.c: */ 
 
@@ -631,6 +684,14 @@ extern void loadrcfile();
 extern short readcommand(short, FILE *);
 #endif
 
+/* from graphic.c */
+
+extern void UDrawGraphic(graphicptr);
+extern Imagedata *addnewimage(char *, int, int);
+extern graphicptr new_graphic(objinstptr, char *, int, int);
+extern void invalidate_graphics(objectptr);
+extern void freegraphic(graphicptr);
+
 /* from render.c: */
 
 extern void ghostinit();
@@ -656,11 +717,12 @@ extern Boolean render_client();
 
 /* from schema.c: */
 
+extern objectptr NameToPageObject(char *, objinstptr *);
 extern objectptr NameToObject(char *, objinstptr *, Boolean);
 extern void callwritenet(xcWidget, pointertype, caddr_t);
 extern void doxschema(xcWidget, pointertype, caddr_t);
 extern void startconnect(xcWidget, caddr_t, caddr_t);
-extern int connectivity(xcWidget, caddr_t, caddr_t);
+extern void connectivity(xcWidget, caddr_t, caddr_t);
 extern Boolean setobjecttype(objectptr);
 extern void pinconvert(labelptr, pointertype);
 extern void dopintype(xcWidget, pointertype, caddr_t);
@@ -682,13 +744,13 @@ extern Boolean schemassoc(objectptr, objectptr);
 
 extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
-extern void geneasydraw(short, short, objectptr, objinstptr);
+extern void geneasydraw(short, int, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
-extern short *genselectelement(short, u_char, objectptr, objinstptr);
+extern selection *genselectelement(short, u_char, objectptr, objinstptr);
 extern short *allocselect();
 extern void setoptionmenu();
 extern int test_insideness(int, int, XPoint *);
-extern Boolean pathselect(genericptr *, short);
+extern Boolean pathselect(genericptr *, short, float);
 extern Boolean areaelement(genericptr *);
 extern void selectarea();
 extern void startdesel(xcWidget, caddr_t, caddr_t);
@@ -698,7 +760,8 @@ extern void draw_all_selected();
 extern void clearselects_noundo();
 extern void clearselects();
 extern void unselect_all();
-extern genericptr recurselect(short, pushlistptr *);
+extern selection *recurselect(short, u_char, pushlistptr *);
+extern short *recurse_select_element(short, u_char);
 #ifdef TCL_WRAPPER
 extern xcTimeOutProc startselect(caddr_t);
 #else
@@ -714,12 +777,14 @@ extern stringpart *nextstringpart(stringpart *, objinstptr);
 extern stringpart *makesegment(stringpart **, stringpart *);
 extern stringpart *splitstring(int, stringpart **, objinstptr);
 extern stringpart *mergestring(stringpart *);
-extern stringpart *linkstring(objinstptr, stringpart *);
+extern stringpart *linkstring(objinstptr, stringpart *, Boolean);
 extern int findcurfont(int, stringpart *, objinstptr);
 extern stringpart *findstringpart(int, int *, stringpart *, objinstptr);
 extern void charprint(char *, stringpart *, int);
 extern char *stringprint(stringpart *, objinstptr);
 extern char *textprint(stringpart *, objinstptr);
+extern char *textprintsubnet(stringpart *, objinstptr, int);
+extern char *textprintnet(char *, char *, Genericlist *);
 extern int textcomp(stringpart *, char *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
@@ -730,6 +795,11 @@ extern stringpart *stringcopy(stringpart *);
 extern stringpart *stringcopyall(stringpart *, objinstptr);
 extern stringpart *stringcopyback(stringpart *, objinstptr);
 extern stringpart *deletestring(stringpart *, stringpart **, objinstptr);
+extern Genericlist *break_up_bus(labelptr, objinstptr, Genericlist *);
+extern int sub_bus_idx(labelptr, objinstptr);
+extern Boolean pin_is_bus(labelptr, objinstptr);
+extern int find_cardinal(int, labelptr, objinstptr);
+extern int find_ordinal(int, labelptr, objinstptr);
 
 extern short UDrawChar(u_char, short, short, int, int);
 extern void UDrawString(labelptr, int, objinstptr);
