@@ -4,6 +4,9 @@
 /* These routines work only if ghostscript is on the system.		*/
 /*----------------------------------------------------------------------*/
 
+/* We use OpenGL functions to read and scale images if OpenGL is enabled */
+#ifndef OPENGL
+
 #undef GS_DEBUG
 /* #define GS_DEBUG */
 
@@ -658,4 +661,6 @@ int reset_gs()
    return 0;
 }
 
+#endif /* !OPENGL */
+
 /*----------------------------------------------------------------------*/
