@@ -216,8 +216,11 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
       /* to a string.  The memory allocated for this string	*/
       /* should be free'd.					*/
 
-      if (strptr->data.string != (u_char *)NULL)
-	 freelabel((stringpart *)(strptr->data.string));
+      if (strptr->data.string != (u_char *)NULL) {
+	 fprintf(stderr, "Non-NULL data in PARAM_END segment\n");
+	 free(strptr->data.string);
+	 strptr->data.string = (u_char *)NULL;
+      }
    }
    return nextptr;
 }
@@ -235,8 +238,11 @@ stringpart *nextstringpartnorecompute(stringpart *strptr, objinstptr thisinst)
       nextptr = linkstring(thisinst, strptr, FALSE);
    else if (strptr->type == PARAM_END) {
       strptr->nextpart = NULL;
-      if (strptr->data.string != (u_char *)NULL)
-	 freelabel((stringpart *)(strptr->data.string));
+      if (strptr->data.string != (u_char *)NULL) {
+	 fprintf(stderr, "Non-NULL data in PARAM_END segment\n");
+	 free(strptr->data.string);
+	 strptr->data.string = (u_char *)NULL;
+      }
    }
    return nextptr;
 }
@@ -267,7 +273,7 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
       /* out if it's an instance or a default, and change the pointer	*/
       /* to the parameter in the parameter list, accordingly.		*/
 
-      else if (strptr->type == PARAM_START) {
+      else if ((strptr->type == PARAM_START) && (thisinst != NULL)) {
 	 key = strptr->data.string;
 	 ops = find_param(thisinst, key);
 	 if (ops == NULL) {
@@ -1168,6 +1174,7 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
       newpart = (stringpart *)malloc(sizeof(stringpart));
       newpart->type = strptr->type;
       newpart->nextpart = NULL;
+      newpart->data.string = NULL;
 
       if (strptr == string) rettop = newpart;	/* initial segment */
       else curend->nextpart = newpart;  	/* append segment to label */
@@ -1715,7 +1722,7 @@ TextExtents ULength(stringpart *string, objinstptr localinst, float newscale,
 
 void undrawtextsimple(labelptr settext)
 {
-   XSetFunction(dpy, areastruct.gc, GXcopy);
+   SetFunction(dpy, areastruct.gc, GXcopy);
    XTopSetForeground(BACKGROUND);
    UDrawString(settext, DOFORALL, areastruct.topinstance);
 }
