@@ -43,12 +43,28 @@
 /* Local structure definitions for holding undo information		*/
 /*----------------------------------------------------------------------*/
 
+typedef struct {
+   float angle1;
+   float angle2;
+   short radius;
+   short yaxis;
+} arcinfo;
+
 typedef struct {
    short number;
    genericptr *element;
    short *idx;
 } uselection;
 
+typedef struct {
+   genericptr element;		/* element being edited */
+   union {
+      stringpart *string;	/* original contents of string, for label */
+      pointlist points;		/* original polygon */
+      arcinfo *arcspecs;	/* original arc values */
+   } save;
+} editelement;
+
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
 /*----------------------------------------------------------------------*/
@@ -141,6 +157,28 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
       return slist;
 }
 
+/*----------------------------------------------------------------------*/
+/* free_editelement ---							*/
+/*									*/
+/*   Free memory allocated to an undo record edit element structure.	*/
+/*----------------------------------------------------------------------*/
+
+void free_editelement(editelement *erec)
+{
+   switch (erec->element->type) {
+      case LABEL:
+	 freelabel(erec->save.string);
+         break;
+      case POLYGON: case SPLINE:
+	 free(erec->save.points);
+	 break;
+      case ARC:
+	 free(erec->save.arcspecs);
+	 break;
+   }
+   free(erec);
+}
+
 /*----------------------------------------------------------------------*/
 /* free_selection ---							*/
 /*									*/
@@ -157,28 +195,31 @@ void free_selection(uselection *selrec)
 }
 
 /*----------------------------------------------------------------------*/
-/* add_to_selectlist ---						*/
+/* get_original_string ---						*/
 /*									*/
-/*   Add an element to the selection list areastruct.selectlist.	*/
+/* Find the original version of the given label.			*/
 /*----------------------------------------------------------------------*/
 
-void add_to_selectlist(genericptr thisinst)
+stringpart *get_original_string(labelptr thislab)
 {
-   int i;
-   short *sptr;
-   genericptr egen;
+   Undoptr chkrecord, thisrecord;
+   editelement *erec;
+   labelptr elab;
 
-   for (i = 0; i < topobject->parts; i++) {
-      egen = *(topobject->plist + i);
-      if (egen == thisinst)
-	 break;
-   }
-   if (i == topobject->parts) {
-      Fprintf(stderr, "Warning:  Element is not in object!\n");
-   }
-   else {
-      sptr = allocselect();
-      *sptr = (short)i;
+   thisrecord = xobjs.undostack;
+   if (thisrecord == NULL) return NULL;
+
+   for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
+      switch (chkrecord->type) {
+	 case XCF_Edit:
+	    erec = (editelement *)(chkrecord->undodata);
+	    elab = (labelptr)(erec->element);
+	    if (elab != thislab) return NULL;
+	    return erec->save.string;
+	    
+	 default:
+	    return NULL;
+      }
    }
 }
 
@@ -218,7 +259,45 @@ int select_previous(Undoptr thisrecord)
 }
 
 /*----------------------------------------------------------------------*/
-/* Put element "thiselem" back into object "thisobj"			*/
+/* This is similar to the above routine, but we just return a pointer	*/
+/* to the index list in the uselection record.  This lets the undelete	*/
+/* function restore the original ordering of parts that were deleted.	*/
+/*----------------------------------------------------------------------*/
+
+short *recover_selectlist(Undoptr thisrecord)
+{
+   Undoptr chkrecord;
+   uselection *srec;
+
+   for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
+
+      /* Selections may cross page changes, but only if in the same series */
+      if ((chkrecord->thisinst != thisrecord->thisinst) &&
+		(chkrecord->idx != thisrecord->idx))
+	 break;
+
+      switch (chkrecord->type) {
+	 case XCF_Delete: case XCF_Pop: case XCF_Push:
+	    /* Delete/Push/Pop records imply a canceled selection */
+	    return NULL;
+	 case XCF_Copy:
+	    /* Copy is the same as Select, but the copied objects are	*/
+	    /* always at the end of the element list.  Returning NULL	*/
+	    /* is the same as declaring that elements should be		*/
+	    /* appended to the end of the object's element list.	*/
+	    return NULL;
+	 case XCF_Select:
+	    srec = (uselection *)chkrecord->undodata;
+	    return srec->idx;
+      }
+   }
+   return NULL;
+}
+
+/*----------------------------------------------------------------------*/
+/* Put element "thiselem" back into object "thisobj".  This is only	*/
+/* used in the case where the elements were previously at the end of	*/
+/* the object's element list.						*/
 /*----------------------------------------------------------------------*/
 
 void undelete_one_element(objinstptr thisinst, genericptr thiselem)
@@ -255,7 +334,8 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    objinstptr newinst;
    Undoptr newrecord;
    uselection *srec;
-   genericptr egen;
+   editelement *erec;
+   genericptr egen, newgen;
    XPoint *fpoint;
 
    /* Do not register new events while running undo/redo actions! */
@@ -348,13 +428,48 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 /* 1 arg:							*/
 	 /*	egen = element just created (genericptr)		*/
 	 egen = va_arg(args, genericptr);
 	 newrecord->undodata = (char *)egen;
 	 break;
 
+      case XCF_Edit:
+	 /* 1 arg:							*/
+	 /*	egen = element to be edited (genericptr)		*/
+	 egen = va_arg(args, genericptr);
+
+	 /* Create a copy of the element to save */
+	 erec = (editelement *)malloc(sizeof(editelement));
+	 erec->element = egen;
+	 switch(egen->type) {
+	    case LABEL:
+	       erec->save.string =
+			stringcopyall(((labelptr)egen)->string,
+			areastruct.topinstance);
+	       newrecord->idata = ((labelptr)egen)->justify;
+	       break;
+	    case POLYGON:
+	       newrecord->idata = ((polyptr)egen)->number;
+	       erec->save.points = copypoints(((polyptr)egen)->points,
+			newrecord->idata);
+	       break;
+	    case SPLINE:
+	       erec->save.points =
+			copypoints((pointlist)((splineptr)egen)->ctrl, 4);
+	       break;
+	    case ARC:
+	       erec->save.arcspecs = (arcinfo *)malloc(sizeof(arcinfo));
+	       erec->save.arcspecs->angle1 = ((arcptr)egen)->angle1;
+	       erec->save.arcspecs->angle2 = ((arcptr)egen)->angle2;
+	       erec->save.arcspecs->radius = ((arcptr)egen)->radius;
+	       erec->save.arcspecs->yaxis = ((arcptr)egen)->yaxis;
+	       break;
+	 }
+	 newrecord->undodata = (char *)erec;
+	 break;
+
       case XCF_Flip_X: case XCF_Flip_Y:
 	 /* 1 arg:							*/
 	 /*	fpoint = point of flip (XPoint *)			*/
@@ -404,6 +519,7 @@ short undo_one_action()
    objectptr thisobj;
    objinstptr thisinst;
    uselection *srec;
+   editelement *erec;
    short *slist;
    XPoint *delta;
    int i, snum, npage, opage;
@@ -416,6 +532,7 @@ short undo_one_action()
       Fprintf(stderr, "Nothing to undo!\n");
       return;
    }
+
    xobjs.undostack = thisrecord->next;
    xobjs.redostack = thisrecord;
 
@@ -432,7 +549,8 @@ short undo_one_action()
 	 thisobj = (objectptr)thisrecord->undodata;
 	 areastruct.selects = thisobj->parts;
 	 areastruct.selectlist = xc_undelete(thisrecord->thisinst,
-			thisobj, (short)thisrecord->idata);
+			thisobj, (short)thisrecord->idata,
+			recover_selectlist(thisrecord));
 	 srec = remember_selection(thisrecord->thisinst, areastruct.selectlist,
 			areastruct.selects);
 	 thisrecord->undodata = (char *)srec;
@@ -461,7 +579,8 @@ short undo_one_action()
 	 /* Put back all the parts */
 	 areastruct.selects = thisobj->parts;
 	 areastruct.selectlist = xc_undelete(thisrecord->thisinst,
-			thisobj, (short)thisrecord->idata);
+			thisobj, (short)thisrecord->idata,
+			recover_selectlist(thisrecord));
 	 srec = remember_selection(thisrecord->thisinst, areastruct.selectlist,
 			areastruct.selects);
 	 thisrecord->undodata = (char *)srec;
@@ -491,7 +610,7 @@ short undo_one_action()
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
 		case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-		case XCF_Spline: case XCF_Dot:
+		case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 egen = (genericptr)thisrecord->undodata;
 	 i = thisrecord->thisinst->thisobject->parts - 1;
 	 if (egen != *(thisrecord->thisinst->thisobject->plist + i)) {
@@ -504,6 +623,65 @@ short undo_one_action()
 	 }
 	 break;
 
+      case XCF_Edit:
+	 erec = (editelement *)thisrecord->undodata;
+	 switch (erec->element->type) {
+	    case LABEL: {
+	       stringpart *tmpstr;
+	       int tmpjust;
+	       labelptr elab = (labelptr)(erec->element);
+	       undrawtext(elab);
+	       tmpstr = elab->string;
+	       tmpjust = (int)elab->justify;
+	       elab->string = stringcopyback(erec->save.string,
+				thisrecord->thisinst);
+	       elab->justify = (short)thisrecord->idata;
+	       erec->save.string = tmpstr;
+	       thisrecord->idata = tmpjust;
+	       resolveparams(thisrecord->thisinst);
+	       redrawtext(elab);
+	    } break;
+	    case POLYGON: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly = (polyptr)(erec->element);
+	       tmppts = epoly->points;
+	       tmpnum = epoly->number;
+	       epoly->points = erec->save.points;
+	       epoly->number = thisrecord->idata;
+	       erec->save.points = tmppts;
+	       thisrecord->idata = tmpnum;
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case SPLINE: {
+	       pointlist tmppts;
+	       splineptr espline = (splineptr)(erec->element);
+	       tmppts = copypoints((pointlist)espline->ctrl, 4);
+	       for (i = 0; i < 4; i++)
+		   espline->ctrl[i] = *(erec->save.points + i);
+	       free(erec->save.points);
+	       erec->save.points = tmppts;
+	       calcspline(espline);
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areastruct.area, NULL, NULL);
+	    }
+	 }
+	 break;
+
       case XCF_Library_Pop:
 	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
@@ -569,9 +747,9 @@ short undo_one_action()
 
 void undo_action()
 {
-   short idx = undo_one_action();
+   short idx = undo_one_action(False);
    while (xobjs.undostack && xobjs.undostack->idx == idx)
-      undo_one_action();
+      undo_one_action(False);
 }
 
 /*----------------------------------------------------------------------*/
@@ -587,7 +765,8 @@ short redo_one_action()
    short *slist;
    XPoint *delta;
    uselection *srec;
-   int tpage, snum;
+   editelement *erec;
+   int i, tpage, snum;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -647,7 +826,7 @@ short redo_one_action()
 	 if (thisobj != NULL) {
 	    unselect_all();
 	    snum = thisobj->parts;
-	    slist = xc_undelete(thisrecord->thisinst, thisobj, DRAW);
+	    slist = xc_undelete(thisrecord->thisinst, thisobj, DRAW, NULL);
 	    thisrecord->undodata = (char *)remember_selection(thisrecord->thisinst,
 			slist, snum);
 	    free(slist);
@@ -659,7 +838,8 @@ short redo_one_action()
 	 if (thisobj != NULL) {
 	    unselect_all();
 	    areastruct.selects = thisobj->parts;
-	    areastruct.selectlist = xc_undelete(thisrecord->thisinst, thisobj, DRAW);
+	    areastruct.selectlist = xc_undelete(thisrecord->thisinst, thisobj, DRAW,
+			NULL);
 	    thisrecord->undodata = (char *)remember_selection(thisrecord->thisinst,
 			areastruct.selectlist, areastruct.selects);
 	    draw_all_selected();
@@ -668,12 +848,71 @@ short redo_one_action()
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 egen = (genericptr)thisrecord->undodata;
 	 undelete_one_element(thisrecord->thisinst, egen);
 	 drawarea(areastruct.area, NULL, NULL);
 	 break;
 
+      case XCF_Edit:
+	 erec = (editelement *)thisrecord->undodata;
+	 switch (erec->element->type) {
+	    case LABEL: {
+	       stringpart *tmpstr;
+	       int tmpjust;
+	       labelptr elab = (labelptr)(erec->element);
+	       undrawtext(elab);
+	       tmpstr = elab->string;
+	       tmpjust = (int)elab->justify;
+	       elab->string = stringcopyback(erec->save.string,
+				thisrecord->thisinst);
+	       elab->justify = (short)thisrecord->idata;
+	       erec->save.string = tmpstr;
+	       thisrecord->idata = tmpjust;
+	       resolveparams(thisrecord->thisinst);
+	       redrawtext(elab);
+	    } break;
+	    case POLYGON: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly = (polyptr)(erec->element);
+	       tmppts = epoly->points;
+	       tmpnum = epoly->number;
+	       epoly->points = erec->save.points;
+	       epoly->number = thisrecord->idata;
+	       erec->save.points = tmppts;
+	       thisrecord->idata = tmpnum;
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case SPLINE: {
+	       pointlist tmppts;
+	       splineptr espline = (splineptr)(erec->element);
+	       tmppts = copypoints((pointlist)espline->ctrl, 4);
+	       for (i = 0; i < 4; i++)
+		   espline->ctrl[i] = *(erec->save.points + i);
+	       free(erec->save.points);
+	       erec->save.points = tmppts;
+	       calcspline(espline);
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areastruct.area, NULL, NULL);
+	    }
+	 }
+	 break;
+
       case XCF_Flip_X:
 	 areastruct.save = *((XPoint *)thisrecord->undodata);
 	 elementflip();
@@ -780,6 +1019,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
    u_int type;
    objectptr uobj;
    uselection *srec;
+   editelement *erec;
 
    type = thisrecord->type;
    switch (type) {
@@ -796,12 +1036,17 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 /* if MODE_UNDO, the element is on the page, so don't destroy it! */
 	 if (mode == MODE_REDO)
 	    free(thisrecord->undodata);
 	 break;
 
+      case XCF_Edit:
+	 erec = (editelement *)thisrecord->undodata;
+	 free_editelement(erec);
+	 break;
+
       case XCF_Copy:
       case XCF_Library_Pop:
 	 if (mode == MODE_UNDO) {
